from fulcrum.utils import is_string, generate_uuid

class Findable(object):
    def find(self, id):
        api_resp = self.client.call('get', '{0}/{1}'.format(self.path, id))
        return api_resp


class Deleteable(object):
    def delete(self, id):
        self.client.call('delete', '{0}/{1}'.format(self.path, id))


class Createable(object):
    def create(self, obj):
        api_resp = self.client.call('post', self.path, data=obj, extra_headers={'Content-Type': 'application/json'})
        return api_resp


class Searchable(object):
    def search(self, url_params=None):
        api_resp = self.client.call('get', self.path, url_params=url_params)
        return api_resp


class Updateable(object):
    def update(self, id, obj):
        api_resp = self.client.call('put', '{0}/{1}'.format(self.path, id), data=obj, extra_headers={'Content-Type': 'application/json'})
        return api_resp


class Media(object):
    def media(self, id, size='original'):
        if size == 'original':
            path = '{}/{}.{}'.format(self.path, id, self.ext)
        else:
            if not size in self.sizes:
                raise ValueError('Size {} not supported'.format(size))
            path = '{}/{}/{}.{}'.format(self.path, id, size, self.ext)

        api_resp = self.client.call('get', path, json_content=False)
        return api_resp


class Track(object):
    track_formats = {
        'json': 'json',
        'geojson': 'geojson',
        'gpx': 'gpx',
        'kml': 'kml',
        'geojson_points': 'geojson?type=points',
    }

    def track(self, id, format='json'):
        if not format in self.track_formats.keys():
            raise ValueError('Format {} not supported'.format(format))
        path = '{}/{}/track.{}'.format(self.path, id, self.track_formats[format])

        is_json_resp = format in ('json', 'geojson', 'geojson_points')

        api_resp = self.client.call('get', path, json_content=is_json_resp)
        return api_resp


class MediaCreateable(object):
    def create(self, media_or_path, content_type=None, access_key=None):
        if is_string(media_or_path):
            media = open(media_or_path, 'rb')
        else:
            media = media_or_path

        data = {
            '{}[access_key]'.format(self.media_form_field_name): access_key or generate_uuid()
        }

        files = {
            '{}[file]'.format(self.media_form_field_name): (media.name, media, content_type or self.default_content_type)
        }

        api_resp = self.client.call('post', self.path + self.media_upload_path, data=data, files=files)
        return api_resp



import sys
import uuid

def is_string(s):
    if sys.version_info[0] == 3:
        return isinstance(s, str)
    else:
        return isinstance(s, basestring)

def generate_uuid():
    return str(uuid.uuid4())



class InvalidAPIVersionException(Exception):
    """An invalid API version was passed."""


class NotFoundException(Exception):
    """The resource could not be found."""


class UnauthorizedException(Exception):
    """The API key supplied is not authorized to access this resource."""


class InternalServerErrorException(Exception):
    """There was an error while processing your request."""


class RateLimitExceededException(Exception):
    """The API key's rate limit was exceeded."""

class BadRequestException(Exception):
    """There was a problem with your request."""



from fulcrum.api import Client
from fulcrum.api.endpoints import (Forms, Records, Webhooks, Photos,
                                   Memberships, Roles, ChoiceLists, Signatures,
                                   ClassificationSets, Projects, Videos, Audio,
                                   Changesets, ChildRecords, AuditLogs, Layers,
                                   Authorizations)
from fulcrum.utils import is_string

__version__ = '1.12.0'

default_uri = 'https://api.fulcrumapp.com'


def create_authorization(email, password, organization_id, note,
                         timeout=None, user_id=None):
    if timeout is not None and not isinstance(timeout, int):
        raise ValueError('timeout must be an integer.')

    if user_id is not None and not is_string(user_id):
        raise ValueError('user_id must be a string.')

    auth = (email, password)
    client = Client(None, default_uri)
    data = {
        'authorization': {
            'organization_id': organization_id,
            'note': note,
            'timeout': timeout,
            'user_id': user_id
        }
    }
    api_resp = client.call('post', 'authorizations', auth=auth, data=data,
                           extra_headers={'Content-Type': 'application/json'})
    return api_resp


def get_user(email, password):
    auth = (email, password)
    client = Client(None, default_uri)
    api_resp = client.call('get', 'users', auth=auth)
    return api_resp


class Fulcrum(object):
    def __init__(self, key, uri=default_uri):
        self.client = Client(key=key, uri=uri)

        self.forms = Forms(client=self.client)
        self.records = Records(client=self.client)
        self.webhooks = Webhooks(client=self.client)
        self.photos = Photos(client=self.client)
        self.signatures = Signatures(client=self.client)
        self.memberships = Memberships(client=self.client)
        self.roles = Roles(client=self.client)
        self.choice_lists = ChoiceLists(client=self.client)
        self.classification_sets = ClassificationSets(client=self.client)
        self.projects = Projects(client=self.client)
        self.videos = Videos(client=self.client)
        self.audio = Audio(client=self.client)
        self.changesets = Changesets(client=self.client)
        self.child_records = ChildRecords(client=self.client)
        self.audit_logs = AuditLogs(client=self.client)
        self.layers = Layers(client=self.client)
        self.authorizations = Authorizations(client=self.client)

    def query(self, sql, format = 'json'):
        obj = {'q': sql, 'format': format}
        kwargs = {
            'data': obj,
            'extra_headers': {'Content-Type': 'application/json'}
        }

        kwargs['json_content'] = False if format == 'csv' else True

        api_resp = self.client.call('post', 'query', **kwargs)
        return api_resp



from fulcrum.mixins import Findable, Deleteable, Createable, Searchable, Updateable, Media, Track, MediaCreateable
from . import BaseAPI


class Forms(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'forms'
    
    def history(self, id, url_params=None):
        api_resp = api_resp = self.client.call('get', '{0}/{1}/history'.format(self.path, id), url_params=url_params)
        return api_resp

class Records(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'records'

    def history(self, id):
        api_resp = api_resp = self.client.call('get', '{0}/{1}/history'.format(self.path, id))
        return api_resp

class Webhooks(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'webhooks'


class Photos(BaseAPI, Findable, Searchable, Media, MediaCreateable):
    path = 'photos'
    ext = 'jpg'
    sizes = ['thumbnail', 'large']
    media_upload_path = ''
    media_form_field_name = 'photo'
    default_content_type = 'image/jpeg'


class Signatures(BaseAPI, Findable, Searchable, Media, MediaCreateable):
    path = 'signatures'
    ext = 'png'
    sizes = ['thumbnail', 'large']
    media_upload_path = ''
    media_form_field_name = 'signature'
    default_content_type = 'image/png'


class Videos(BaseAPI, Findable, Searchable, Media, Track, MediaCreateable):
    path = 'videos'
    ext = 'mp4'
    sizes = ['small', 'medium']
    media_upload_path = '/upload'
    media_form_field_name = 'video'
    default_content_type = 'video/mp4'


class Audio(BaseAPI, Findable, Searchable, Media, Track, MediaCreateable):
    path = 'audio'
    ext = 'mp4'
    sizes = []
    media_upload_path = '/upload'
    media_form_field_name = 'audio'
    default_content_type = 'audio/mp3'


class Memberships(BaseAPI, Searchable):
    path = 'memberships'

    def change(self, resource_type, id, action, membership_ids):
        change = {
            'type': '{}_members'.format(resource_type),
            '{}_id'.format(resource_type): id,
            action: membership_ids
        }
        data = {'change': change}
        api_resp = self.client.call('post', 'memberships/change_permissions',
                                    data=data,
                                    extra_headers={'Content-Type': 'application/json'})
        return api_resp


class Roles(BaseAPI, Searchable):
    path = 'roles'


class ChoiceLists(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'choice_lists'


class ClassificationSets(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'classification_sets'


class Projects(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'projects'


class Changesets(BaseAPI, Findable, Createable, Searchable, Updateable):
    path = 'changesets'

    def close(self, id):
        api_resp = api_resp = self.client.call('put', '{0}/{1}/close'.format(self.path, id))
        return api_resp


class ChildRecords(BaseAPI, Searchable):
    path = 'child_records'


class AuditLogs(BaseAPI, Searchable, Findable):
    path = 'audit_logs'


class Layers(BaseAPI, Findable, Deleteable, Createable, Searchable, Updateable):
    path = 'layers'


class Authorizations(BaseAPI, Findable, Deleteable, Searchable, Updateable):
    path = 'authorizations'

    def regenerate(self, id):
        api_resp = self.client.call('post', '{}/{}/regenerate'.format(self.path, id))
        return api_resp



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

requires = ['requests>=2.11.0']

setup(
    name='fulcrum',
    version='1.12.0',
    description='A python wrapper for the Fulcrum API',
    author='Jason Sanford',
    author_email='jasonsanford@gmail.com',
    url='https://github.com/fulcrumapp/fulcrum-python',
    packages= ['fulcrum', 'fulcrum.api'],
    install_requires=requires,
    license='Apache License',
)



import json

import requests

import fulcrum
from fulcrum.exceptions import (NotFoundException, UnauthorizedException,
                                InternalServerErrorException,
                                RateLimitExceededException, BadRequestException)


class Client(object):
    http_exception_map = {
        400: BadRequestException,
        401: UnauthorizedException,
        404: NotFoundException,
        429: RateLimitExceededException,
        500: InternalServerErrorException,
    }

    def __init__(self, key, uri):
        self.key = key
        self.api_root = '{0}/api/v2/'.format(uri)

    def call(self, method, path, data=None, extra_headers=None, url_params=None, json_content=True, files=None, auth=None):
        full_path = self.api_root + path

        headers = {
            'User-Agent': 'Fulcrum Python API Client, Version {}'.format(fulcrum.__version__),
        }

        if self.key:
            headers['X-ApiToken'] = self.key

        if json_content:
            headers.update({'Accept': 'application/json'})

        if extra_headers is not None:
            headers.update(extra_headers)

        kwargs = {'headers': headers}

        if data is not None:
            if files:
                kwargs['data'] = data
            else:
                kwargs['data'] = json.dumps(data)

        if url_params is not None:
            kwargs['params'] = url_params

        if files is not None:
            kwargs['files'] = files

        if auth is not None:
            kwargs['auth'] = auth

        resp = getattr(requests, method)(full_path, **kwargs)

        if resp.status_code in self.http_exception_map:
            raise self.http_exception_map[resp.status_code]

        if method == 'delete' or (method == 'put' and 'close' in path):
            # No body is returned for delete and close methods.
            return
        elif json_content:
            return resp.json()
        else:
            return resp.content


class BaseAPI(object):
    def __init__(self, client):
        self.client = client
